#ifndef WATER_JUGS_HPP_
#define WATER_JUGS_HPP_


#include "array_set.hpp"
#include "array_queue.hpp"
#include "array_priority_queue.hpp"
#include "array_map.hpp"


namespace ics {


class Operator;
class Problem;


class State {
public:
  State();
  State(const State& s);
  State(int jugs[], Problem* prob);
  virtual ~State();
  int                  how_close    () const;
  State&               operator =   (const State& rhs);
  bool                 operator ==  (const State& rhs) const;
  friend std::ostream& operator <<  (std::ostream& outs, const State& s);
  friend Operator;
private:
  int*     jug_state;
  Problem* problem;
};



class Operator {
public:
  Operator();
  Operator(int jug_from, int jug_to, int jug_count);
  virtual ~Operator();
  State                apply        (const State& s) const;
  State                copy         () const;
  bool                 operator ==  (const Operator& rhs) const;
  friend std::ostream& operator <<  (std::ostream& outs, const Operator& o);
private:
  int from;
  int to;
};



class Problem {
public:
  Problem(int all_jug_sizes[], int start[], int stop[], int length);
  virtual ~Problem();
  State                   get_start_state   () const;
  State                   get_stop_state    () const;
  ics::ArraySet<Operator> get_all_operators () const;
  friend std::ostream& operator <<  (std::ostream& outs, const Problem& p);
  friend std::ostream& operator <<  (std::ostream& outs, const State& s);
  friend State;
  friend Operator;

  template<class Queue>
  friend ics::ArrayQueue<ics::Operator> solve_it (ics::Problem& problem,
                                                  Queue&& exploring);

private:
  int   length;
  int*  jug_sizes;
  State start;
  State stop;
  ics::ArrayMap<ics::State,ics::ArrayQueue<ics::Operator>> solutions;
};

}

#endif /* WATER_JUGS_HPP_ */
