#include "water_jugs.hpp"
#include "solver.hpp"
#include "stopwatch.hpp"


static bool best_compare(const ics::State& a, const ics::State& b) {
  return a.how_close() < b.how_close();
}


int main() {
//  try {
    int sizes[] = {5,11,13,24};  //See problem below
    int start[] = {0,0,0,24};
    int stop [] = {0,8,8,8};
    ics::Problem problem(sizes,start,stop,4);

//    int sizes[] = {3,5,8};       //See lecture notes
//    int start[] = {0,0,8};
//    int stop [] = {0,4,4};
//    ics::Problem problem(sizes,start,stop,3);

    //Show all operators in the problem
    std::cout << problem << std::endl;
    std::cout << "Operators for problem" << std::endl;
    for (ics::Operator o : problem.get_all_operators())
      std::cout << "  " << o << std::endl;
    std::cout << std::endl;

    //Try solution via breadth-first tree traversal
    std::cout << "Breadth-first solution" << std::endl;
    ics::Stopwatch timeit;
    timeit.start();
    ics::ArrayQueue<ics::Operator> solution = ics::breadth_first_solution(problem);
    timeit.stop();
    std::cout << "Time spent searching for breadth-first solution = " << timeit.read() << std::endl;
    std::cout << ics::illustrate_solution(problem.get_start_state(),solution) << std::endl;

    //Try solution via best-first tree traversal
    std::cout << "\nBest-first solution" << std::endl;
    timeit.reset();
    timeit.start();
    solution = ics::best_first_solution(problem, best_compare);
    timeit.stop();
    std::cout << "Time spent searching for best-first solution = " << timeit.read() << std::endl;
    std::cout << ics::illustrate_solution(problem.get_start_state(),solution) << std::endl;
//  }
//  catch (std::string& s) {std::cout << s << std::endl;}
}




/*
Problem from http://www.cut-the-knot.org/ctk/Water.shtml

Three men robbed a gentleman of a vase, containing 24 ounces of balsam.
Whilst running away they met a glass seller, of whom they purchased
three vessels. On reaching a place of safety they wished to divide the
booty, but found that their vessels could hold 5, 11, and 13 ounces respectively.
How could they divide the balsam into equal portions?

*/
